(define-module (ordo play)
  #:use-module (ice-9 match)
  #:use-module (srfi srfi-1) ; list utils
  #:use-module (srfi srfi-9) ; records
  #:use-module (srfi srfi-26) ; cut
  #:use-module (ordo connection)
  #:use-module (ordo context)
  #:use-module (ordo task)
  #:use-module (ordo handler)
  #:export (play run-play))

(define-record-type <play>
  (make-play description connection vars tasks handlers)
  play?
  (connection play-connection)
  (vars play-vars)
  (description play-description)
  (tasks play-tasks)
  (handlers play-handlers))

(define* (play description #:key connection tasks (vars '()) (handlers '()))
  ;; TODO: validation could be better - check for non-empty tasks list, check
  ;; type of connection, tasks, and handlers, etc.
  (unless connection (error "connection is required"))
  (unless tasks (error "tasks are required"))
  (for-each (lambda (task)
              (for-each (lambda (trigger)
                          (unless (assoc-ref handlers trigger)
                            (error (format #f "task \"~a\" references an undefined trigger: ~a"
                                           (task-description task)
                                           trigger))))
                        (task-triggers task)))
            tasks)
  (make-play description connection vars tasks handlers))

(define (run-trigger conn ctx handlers trigger)
  (let ((h (assoc-ref handlers trigger)))
    (unless h
      (error (format #f "no handler defined for trigger ~a" trigger)))
    (run-handler conn ctx h)))

(define (run-play play)
  (format #t "RUNNING PLAY ~a~%" (play-description play))
  (call-with-connection
   (play-connection play)
   (lambda (c)
     (let* ((tmp-dir (car (connection-must c "mktemp" '("--directory"))))
            (ctx (context #:scratch-dir tmp-dir #:init-vars (play-vars play))))
       (dynamic-wind
         (const #t)
         (lambda ()
           (for-each (cut run-task c ctx <>) (play-tasks play))
           (for-each (match-lambda
                       ((name . handler)
                        (when (context-triggered? ctx name)
                          (run-handler c ctx handler))))
                     (play-handlers play)))
         (lambda () (connection-must c "rm" `("-rf" ,tmp-dir))))))))
